# -*- coding: utf-8 -*-
"""
    crimson
    ~~~~~~~

    :copyright: (c) 2015 Wibowo Arindrarto <bow@bow.web.id>
    :license: BSD

"""

RELEASE = True

__version_info__ = ("0", "2", "0")
__version__ = ".".join(__version_info__)
__version__ += "-dev" if not RELEASE else ""

__author__ = "Wibowo Arindrarto"
__contact__ = "bow@bow.web.id"
__homepage__ = "http://bow.web.id"
